//
// Shige-Fuji Project
// SwiftUI Archives
//
// Copyright (c) Watanabe-Denki, Inc.
//   https://wdkk.co.jp/
//
// This software is released under the MIT License.
//   https://opensource.org/licenses/mit-license.php
//

import UIKit
import RealityKit

class CustomARView: ARView {
    // アンカー
    var firstAnchor: AnchorEntity = AnchorEntity()
    
    required init(frame frameRect: CGRect) {
        super.init(frame: frameRect)
        createScene()
    }
    
    @MainActor required dynamic init?(coder decoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func prepareAnchor() {
        // 検出無しに設置するアンカー
        firstAnchor.position = simd_make_float3(0.0, -0.5, -1.0) // アンカーの位置は、デバイス初期位置から0.5m下、1m奥（ARKitでは1m = 1.0）
        self.scene.anchors.append(firstAnchor)   // CustomARView(self)にアンカーを追加
    }
    
    func createScene() {
        // アンカーの準備
        prepareAnchor()
        
        // PhysicallyBasedMaterial: 物理演算有りのマテリアル
        var phys_material = PhysicallyBasedMaterial()
        // マテリアルの色
        phys_material.baseColor = PhysicallyBasedMaterial.BaseColor(tint: .systemBlue)
        // クリアコート（光沢感）の感度
        //phys_material.clearcoat = 1.0
        // クリアコートサーフェスの視覚的な粗さ（滑らか 0.0~1.0 粗い）
        //phys_material.clearcoatRoughness = 0.0
        // サーフェスの質感の調整。滑らで鏡のように反射するか、質感か粗く光沢感がなくなるか（滑らか 0.0~1.0 粗い）
        phys_material.roughness = 0.2
        // 金属っぽく見えるか（誘導体 0.0~1.0 金属）
        phys_material.metallic = 1.0
        // 誘導体のとき、サーフェスの反射を無反射から全反射まで設定できる（無反射 0.0~1.0 全反射）
        //phys_material.specular = 0.5
   
        // 立方体を作成
        let box_phys = ModelEntity(mesh: .generateBox(size: simd_make_float3(0.3, 0.3, 0.3)))
        box_phys.model?.materials = [phys_material]
        box_phys.transform = Transform(translation: simd_make_float3(0.0, 0.2, 0.2))
        self.firstAnchor.addChild(box_phys)     // アンカーに追加
    }
}

